package org.jboss.qa.brms.performance;

import org.drools.KnowledgeBase;
import org.drools.KnowledgeBaseConfiguration;
import org.drools.KnowledgeBaseFactory;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.ResourceType;
import org.drools.conf.EventProcessingOption;
import org.drools.io.impl.ClassPathResource;
import org.drools.runtime.KnowledgeSessionConfiguration;
import org.drools.runtime.StatefulKnowledgeSession;
import org.drools.runtime.conf.ClockTypeOption;
import org.drools.runtime.rule.WorkingMemoryEntryPoint;
import org.jboss.qa.brms.performance.domain.NewCustomerEvent;
import org.jboss.qa.brms.performance.domain.TransactionCreatedEvent;
import org.junit.Test;

import javax.xml.transform.sax.SAXSource;
import java.util.ArrayList;
import java.util.List;

public class SimpleAfterBeforeOperatorsTest {
    private static int LOOPS = 20;
    private static int NR_OF_EVENTS = 1000;

    private KnowledgeBase kbase;

    @Test
    public void testCreateKBaseInsertFactsAndFireRules() {
        createKBase("fusion-after-before-operators-100.drl");
        for (int i = 0; i < LOOPS; i++) {
            executeOneLoop(i);
        }
    }

    private void createKBase(String drlName) {
        KnowledgeBuilder kbuilder = KnowledgeBuilderFactory.newKnowledgeBuilder();
        kbuilder.add(new ClassPathResource(drlName), ResourceType.DRL);
        KnowledgeBaseConfiguration kbconf = KnowledgeBaseFactory.newKnowledgeBaseConfiguration();
        String modeOption = System.getProperty("drools.mode");
        if (modeOption != null && modeOption.equals("stream")) {
            kbconf.setOption(EventProcessingOption.STREAM);
            System.out.println("Using STREAM mode.");
        } else {
            kbconf.setOption(EventProcessingOption.CLOUD);
            System.out.println("Using CLOUD mode.");
        }

        if (kbuilder.hasErrors()) {
            throw new RuntimeException(kbuilder.getErrors().toString());
        }

        kbase = KnowledgeBaseFactory.newKnowledgeBase(kbconf);
        kbase.addKnowledgePackages(kbuilder.getKnowledgePackages());
    }

    private void executeOneLoop(int loopNumber) {
        StatefulKnowledgeSession ksession = createStatefulKSession();
        WorkingMemoryEntryPoint entryPoint = ksession.getWorkingMemoryEntryPoint("Master Stream");
        long start = System.nanoTime();
        for (Object event : getEvents(NR_OF_EVENTS)) {
            entryPoint.insert(event);
        }
        ksession.fireAllRules();
        long took = System.nanoTime() - start;
        System.out.println("Loop #" + loopNumber + "took " + took / 1000 / 1000 + "ms.");
        ksession.dispose();
        System.gc();
    }

    private StatefulKnowledgeSession createStatefulKSession() {
        KnowledgeSessionConfiguration ksconf = KnowledgeBaseFactory.newKnowledgeSessionConfiguration();
        ksconf.setOption(ClockTypeOption.get("realtime"));
        StatefulKnowledgeSession ksession = kbase.newStatefulKnowledgeSession(ksconf, null);
        return ksession;
    }

    private List<Object> getEvents(int nrOfEvents) {
        List<Object> events = new ArrayList<Object>();
        for (int i = 0; i < nrOfEvents / 2; i++) {
            NewCustomerEvent customerEvent = new NewCustomerEvent(i);
            customerEvent.setDescription("AfterBefore_TransactionCreatedAfterNewCustomerCreated");
            events.add(customerEvent);
            TransactionCreatedEvent transactionCreatedEvent = new TransactionCreatedEvent(i);
            transactionCreatedEvent.setDescription("AfterBefore_TransactionCreatedBeforeNewCustomerCreated");
            events.add(transactionCreatedEvent);
        }
        return events;
    }

}
